@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2023 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2023 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <string.h>
 #include <stdio.h>
@@ -55,6 +35,10 @@
  * - a new virtual sink that forwards audio to other sinks
  * - a new virtual source that combines audio from other sources
  *
+ * The sources and sink that need to be combined can be selected using generic match
+ * rules. This makes it possible to combine static nodes or nodes based on certain
+ * properties.
+ *
  * ## Module Options
  *
  * - `node.name`: a unique name for the stream
@@ -62,6 +46,7 @@
  * - `combine.mode` = capture | playback | sink | source, default sink
  * - `combine.props = {}`: properties to be passed to the sink/source
  * - `stream.props = {}`: properties to be passed to the streams
+ * - `stream.rules = {}`: rules for matching streams, use create-stream actions
  *
  * ## General options
  *
@@ -218,15 +203,15 @@ PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
 #define DEFAULT_CHANNELS 2
 #define DEFAULT_POSITION "[ FL FR ]"
 
-#define MODULE_USAGE	"[ node.latency=<latency as fraction> ] "				\
-			"[ combine.mode=<mode of stream, playback|capture|sink|source>, default:sink ] "	\
-			"[ node.name=<name of the stream> ] "					\
-			"[ node.description=<description of the stream> ] "			\
-			"[ audio.channels=<number of channels, default:"SPA_STRINGIFY(DEFAULT_CHANNELS) "> ] "	\
-			"[ audio.position=<channel map, default:"DEFAULT_POSITION"> ] "		\
-			"[ combine.props=<properties> ] "						\
-			"[ stream.props=<properties> ] "					\
-			"[ stream.rules=<properties> ] "
+#define MODULE_USAGE	"( node.latency=<latency as fraction> ) "				\
+			"( combine.mode=<mode of stream, playback|capture|sink|source>, default:sink ) "	\
+			"( node.name=<name of the stream> ) "					\
+			"( node.description=<description of the stream> ) "			\
+			"( audio.channels=<number of channels, default:"SPA_STRINGIFY(DEFAULT_CHANNELS) "> ) "	\
+			"( audio.position=<channel map, default:"DEFAULT_POSITION"> ) "		\
+			"( combine.props=<properties> ) "					\
+			"( stream.props=<properties> ) "					\
+			"( stream.rules=<properties> ) "
 
 
 static const struct spa_dict_item module_props[] = {
@@ -596,7 +581,7 @@ static void registry_event_global(void *data, uint32_t id,
 		if (impl->mode == MODE_CAPTURE || impl->mode == MODE_SINK)
 			str = "[ { matches = [ { media.class = \"Audio/Sink\" } ] "
 				"  actions = { create-stream = {} } } ]";
-		else if (impl->mode == MODE_PLAYBACK || impl->mode == MODE_SOURCE)
+		else
 			str = "[ { matches = [ { media.class = \"Audio/Source\" } ] "
 				"  actions = { create-stream = {} } } ]";
 	}
@@ -847,6 +832,7 @@ static void core_removed(void *d)
 }
 
 static const struct pw_proxy_events core_proxy_events = {
+	PW_VERSION_PROXY_EVENTS,
 	.removed = core_removed,
 };
 
