@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2021 Georges Basile Stavracas Neto
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2021 Georges Basile Stavracas Neto */
+/* SPDX-License-Identifier: MIT */
 
 #include <pipewire/pipewire.h>
 
@@ -60,7 +40,7 @@ static void module_null_sink_proxy_destroy(void *data)
 	module_schedule_unload(module);
 }
 
-static void module_null_sink_proxy_bound(void *data, uint32_t global_id)
+static void module_null_sink_proxy_bound_props(void *data, uint32_t global_id, const struct spa_dict *props)
 {
 	struct module *module = data;
 	struct module_null_sink_data *d = module->user_data;
@@ -83,7 +63,7 @@ static void module_null_sink_proxy_error(void *data, int seq, int res, const cha
 static const struct pw_proxy_events proxy_events = {
 	PW_VERSION_PROXY_EVENTS,
 	.removed = module_null_sink_proxy_removed,
-	.bound = module_null_sink_proxy_bound,
+	.bound_props = module_null_sink_proxy_bound_props,
 	.error = module_null_sink_proxy_error,
 	.destroy = module_null_sink_proxy_destroy,
 };
@@ -183,6 +163,19 @@ static int module_null_sink_prepare(struct module * const module)
 	if (module_args_to_audioinfo(module->impl, props, &info) < 0)
 		return -EINVAL;
 
+	info.format = module->impl->defs.sample_spec.format;
+	if ((str = pw_properties_get(props, "format")) != NULL) {
+		info.format = format_paname2id(str, strlen(str));
+		if (info.format == SPA_AUDIO_FORMAT_UNKNOWN) {
+			pw_log_error("invalid format '%s'", str);
+			return -EINVAL;
+		}
+		pw_properties_set(props, "format", NULL);
+	}
+
+	if (info.format)
+		pw_properties_setf(props, SPA_KEY_AUDIO_FORMAT, "%s",
+					format_id2name(info.format));
 	if (info.rate)
 		pw_properties_setf(props, SPA_KEY_AUDIO_RATE, "%u", info.rate);
 	if (info.channels) {
