@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2021 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2021 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <string.h>
 #include <stdio.h>
@@ -52,6 +32,7 @@
 #include <pipewire/private.h>
 
 #include <pulse/pulseaudio.h>
+#include "module-protocol-pulse/defs.h"
 #include "module-protocol-pulse/format.h"
 
 /** \page page_module_pulse_tunnel PipeWire Module: Pulse Tunnel
@@ -125,19 +106,19 @@ PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
 #define DEFAULT_CHANNELS 2
 #define DEFAULT_POSITION "[ FL FR ]"
 
-#define MODULE_USAGE	"[ remote.name=<remote> ] "				\
-			"[ node.latency=<latency as fraction> ] "		\
-			"[ node.name=<name of the nodes> ] "			\
-			"[ node.description=<description of the nodes> ] "	\
-			"[ node.target=<remote node target name or serial> ] "	\
-			"[ audio.format=<sample format> ] "			\
-			"[ audio.rate=<sample rate> ] "				\
-			"[ audio.channels=<number of channels> ] "		\
-			"[ audio.position=<channel map> ] "			\
+#define MODULE_USAGE	"( remote.name=<remote> ] "				\
+			"( node.latency=<latency as fraction> ] "		\
+			"( node.name=<name of the nodes> ] "			\
+			"( node.description=<description of the nodes> ] "	\
+			"( node.target=<remote node target name or serial> ] "	\
+			"( audio.format=<sample format> ] "			\
+			"( audio.rate=<sample rate> ] "				\
+			"( audio.channels=<number of channels> ] "		\
+			"( audio.position=<channel map> ] "			\
 			"pulse.server.address=<address> "			\
-			"pulse.latency=<latency in msec> "			\
-			"[ tunnel.mode=source|sink "				\
-			"[ stream.props=<properties> ] "
+			"( pulse.latency=<latency in msec, default 200> ) "	\
+			"( tunnel.mode=source|sink, default sink ) "				\
+			"( stream.props=<properties> ) "
 
 
 static const struct spa_dict_item module_props[] = {
@@ -154,6 +135,7 @@ static const struct spa_dict_item module_props[] = {
 
 struct impl {
 	struct pw_context *context;
+	struct pw_loop *main_loop;
 
 #define MODE_SINK	0
 #define MODE_SOURCE	1
@@ -180,9 +162,13 @@ struct impl {
 	void *buffer;
 	uint8_t empty[8192];
 
+	bool mute;
+	pa_cvolume volume;
+
 	pa_threaded_mainloop *pa_mainloop;
 	pa_context *pa_context;
 	pa_stream *pa_stream;
+	uint32_t pa_index;
 
 	struct ratelimit rate_limit;
 
@@ -251,23 +237,95 @@ static void stream_state_changed(void *d, enum pw_stream_state old,
 	}
 }
 
-static void update_rate(struct impl *impl, bool playback)
+static void stream_param_changed(void *d, uint32_t id, const struct spa_pod *param)
+{
+	struct impl *impl = d;
+	char buf[1024];
+	struct spa_pod_builder b = SPA_POD_BUILDER_INIT(buf, sizeof(buf));
+	struct spa_pod_frame f[1];
+	struct spa_pod_object *obj = (struct spa_pod_object *) param;
+	struct spa_pod_prop *prop;
+
+	if (param == NULL || id != SPA_PARAM_Props)
+		return;
+
+	spa_pod_builder_push_object(&b, &f[0], SPA_TYPE_OBJECT_Props, SPA_PARAM_Props);
+
+	SPA_POD_OBJECT_FOREACH(obj, prop) {
+		switch (prop->key) {
+		case SPA_PROP_mute:
+		{
+			bool mute;
+			if (spa_pod_get_bool(&prop->value, &mute) == 0) {
+				pa_threaded_mainloop_lock(impl->pa_mainloop);
+				if (impl->mode == MODE_SOURCE) {
+					pa_context_set_source_output_mute(impl->pa_context,
+							impl->pa_index, mute,
+							NULL, impl);
+				} else {
+					pa_context_set_sink_input_mute(impl->pa_context,
+							impl->pa_index, mute,
+							NULL, impl);
+				}
+				pa_threaded_mainloop_unlock(impl->pa_mainloop);
+                        }
+			break;
+		}
+		case SPA_PROP_channelVolumes:
+		{
+			struct pa_cvolume volume;
+			uint32_t n;
+			float vols[SPA_AUDIO_MAX_CHANNELS];
+
+			if ((n = spa_pod_copy_array(&prop->value, SPA_TYPE_Float,
+					vols, SPA_AUDIO_MAX_CHANNELS)) > 0) {
+				volume.channels = n;
+				for (n = 0; n < volume.channels; n++)
+					volume.values[n] = pa_sw_volume_from_linear(vols[n]);
+
+				pa_threaded_mainloop_lock(impl->pa_mainloop);
+				if (impl->mode == MODE_SOURCE) {
+					pa_context_set_source_output_volume(impl->pa_context,
+							impl->pa_index, &volume,
+							NULL, impl);
+				} else {
+					pa_context_set_sink_input_volume(impl->pa_context,
+							impl->pa_index, &volume,
+							NULL, impl);
+				}
+				pa_threaded_mainloop_unlock(impl->pa_mainloop);
+			}
+			break;
+		}
+		case SPA_PROP_softVolumes:
+		case SPA_PROP_softMute:
+			break;
+		default:
+			spa_pod_builder_raw_padded(&b, prop, SPA_POD_PROP_SIZE(prop));
+			break;
+		}
+	}
+	param = spa_pod_builder_pop(&b, &f[0]);
+
+	pw_stream_set_param(impl->stream, id, param);
+}
+
+static void update_rate(struct impl *impl, uint32_t filled)
 {
 	float error, corr;
+	uint32_t current_latency;
 
 	if (impl->rate_match == NULL)
 		return;
 
-	if (playback)
-		error = (float)impl->target_latency - (float)impl->current_latency;
-	else
-		error = (float)impl->current_latency - (float)impl->target_latency;
+	current_latency = impl->current_latency + filled;
+	error = (float)impl->target_latency - (float)(current_latency);
 	error = SPA_CLAMP(error, -impl->max_error, impl->max_error);
 
 	corr = spa_dll_update(&impl->dll, error);
 	pw_log_debug("error:%f corr:%f current:%u target:%u",
 			error, corr,
-			impl->current_latency, impl->target_latency);
+			current_latency, impl->target_latency);
 
 	SPA_FLAG_SET(impl->rate_match->flags, SPA_IO_RATE_MATCH_FLAG_ACTIVE);
 	impl->rate_match->rate = 1.0f / corr;
@@ -302,7 +360,7 @@ static void playback_stream_process(void *d)
                                         size, RINGBUFFER_SIZE);
 		impl->resync = true;
 	} else {
-		update_rate(impl, true);
+		update_rate(impl, filled / impl->frame_size);
 	}
 	spa_ringbuffer_write_data(&impl->ring,
 				impl->buffer, RINGBUFFER_SIZE,
@@ -343,7 +401,7 @@ static void capture_stream_process(void *d)
 			avail = impl->target_buffer;
 			index += avail - impl->target_buffer;
 		} else {
-			update_rate(impl, false);
+			update_rate(impl, avail / impl->frame_size);
 		}
 		spa_ringbuffer_read_data(&impl->ring,
 				impl->buffer, RINGBUFFER_SIZE,
@@ -375,6 +433,7 @@ static const struct pw_stream_events playback_stream_events = {
 	.destroy = stream_destroy,
 	.state_changed = stream_state_changed,
 	.io_changed = stream_io_changed,
+	.param_changed = stream_param_changed,
 	.process = playback_stream_process
 };
 
@@ -383,6 +442,7 @@ static const struct pw_stream_events capture_stream_events = {
 	.destroy = stream_destroy,
 	.state_changed = stream_state_changed,
 	.io_changed = stream_io_changed,
+	.param_changed = stream_param_changed,
 	.process = capture_stream_process
 };
 
@@ -435,6 +495,20 @@ static int create_stream(struct impl *impl)
 	return 0;
 }
 
+static int
+do_schedule_destroy(struct spa_loop *loop,
+	bool async, uint32_t seq, const void *data, size_t size, void *user_data)
+{
+	struct impl *impl = user_data;
+	pw_impl_module_schedule_destroy(impl->module);
+	return 0;
+}
+
+void module_schedule_destroy(struct impl *impl)
+{
+	pw_loop_invoke(impl->main_loop, do_schedule_destroy, 1, NULL, 0, false, impl);
+}
+
 static void context_state_cb(pa_context *c, void *userdata)
 {
 	struct impl *impl = userdata;
@@ -456,7 +530,7 @@ static void context_state_cb(pa_context *c, void *userdata)
 		break;
 	}
 	if (do_destroy)
-		pw_impl_module_schedule_destroy(impl->module);
+		module_schedule_destroy(impl);
 }
 
 static void stream_state_cb(pa_stream *s, void * userdata)
@@ -469,6 +543,7 @@ static void stream_state_cb(pa_stream *s, void * userdata)
 		do_destroy = true;
 		SPA_FALLTHROUGH;
 	case PA_STREAM_READY:
+		impl->pa_index = pa_stream_get_index(impl->pa_stream);
 		pa_threaded_mainloop_signal(impl->pa_mainloop, 0);
 		break;
 	case PA_STREAM_UNCONNECTED:
@@ -478,7 +553,7 @@ static void stream_state_cb(pa_stream *s, void * userdata)
 		break;
 	}
 	if (do_destroy)
-		pw_impl_module_schedule_destroy(impl->module);
+		module_schedule_destroy(impl);
 }
 
 static void stream_read_request_cb(pa_stream *s, size_t length, void *userdata)
@@ -531,7 +606,6 @@ static void stream_read_request_cb(pa_stream *s, size_t length, void *userdata)
 
 	pa_stream_get_latency(impl->pa_stream, &latency, &negative);
 	impl->current_latency = latency * impl->info.rate / SPA_USEC_PER_SEC;
-	impl->current_latency += filled / impl->frame_size;
 
 	spa_ringbuffer_write_update(&impl->ring, index);
 }
@@ -556,7 +630,6 @@ static void stream_write_request_cb(pa_stream *s, size_t length, void *userdata)
 
 	pa_stream_get_latency(impl->pa_stream, &latency, &negative);
 	impl->current_latency = latency * impl->info.rate / SPA_USEC_PER_SEC;
-	impl->current_latency += avail / impl->frame_size;
 
 	while (avail < (int32_t)length) {
 		uint32_t maxsize = SPA_ROUND_DOWN(sizeof(impl->empty), impl->frame_size);
@@ -622,6 +695,78 @@ static void stream_latency_update_cb(pa_stream *s, void *userdata)
 	pa_threaded_mainloop_signal(impl->pa_mainloop, 0);
 }
 
+static int
+do_stream_sync_volumes(struct spa_loop *loop,
+	bool async, uint32_t seq, const void *data, size_t size, void *user_data)
+{
+	struct impl *impl = user_data;
+	char buf[1024];
+	struct spa_pod_builder b = SPA_POD_BUILDER_INIT(buf, sizeof(buf));
+	struct spa_pod_frame f[1];
+	struct spa_pod *param;
+	uint32_t i;
+	float vols[SPA_AUDIO_MAX_CHANNELS];
+	float soft_vols[SPA_AUDIO_MAX_CHANNELS];
+
+	for (i = 0; i < impl->volume.channels; i++) {
+		vols[i] = pa_sw_volume_to_linear(impl->volume.values[i]);
+		soft_vols[i] = 1.0f;
+	}
+
+	spa_pod_builder_push_object(&b, &f[0], SPA_TYPE_OBJECT_Props, SPA_PARAM_Props);
+	spa_pod_builder_prop(&b, SPA_PROP_softMute, 0);
+	spa_pod_builder_bool(&b, impl->mute);
+	spa_pod_builder_prop(&b, SPA_PROP_mute, 0);
+	spa_pod_builder_bool(&b, impl->mute);
+
+	spa_pod_builder_prop(&b, SPA_PROP_channelVolumes, 0);
+	spa_pod_builder_array(&b, sizeof(float), SPA_TYPE_Float,
+			impl->volume.channels, vols);
+	spa_pod_builder_prop(&b, SPA_PROP_softVolumes, 0);
+	spa_pod_builder_array(&b, sizeof(float), SPA_TYPE_Float,
+			impl->volume.channels, soft_vols);
+	param = spa_pod_builder_pop(&b, &f[0]);
+
+	pw_stream_set_param(impl->stream, SPA_PARAM_Props, param);
+	return 0;
+}
+
+static void stream_sync_volumes(struct impl *impl, const struct pa_cvolume *volume, bool mute)
+{
+	impl->mute = mute;
+	impl->volume = *volume;
+	pw_loop_invoke(impl->main_loop, do_stream_sync_volumes, 1, NULL, 0, false, impl);
+}
+
+static void source_output_info_cb(pa_context *c, const pa_source_output_info *i, int eol, void *userdata)
+{
+	struct impl *impl = userdata;
+	if (i != NULL)
+		stream_sync_volumes(impl, &i->volume, i->mute);
+}
+
+static void sink_input_info_cb(pa_context *c, const pa_sink_input_info *i, int eol, void *userdata)
+{
+	struct impl *impl = userdata;
+	if (i != NULL)
+		stream_sync_volumes(impl, &i->volume, i->mute);
+}
+
+static void context_subscribe_cb(pa_context *c, pa_subscription_event_type_t t,
+		uint32_t idx, void *userdata)
+{
+	struct impl *impl = userdata;
+	if (idx != impl->pa_index)
+		return;
+
+	if (impl->mode == MODE_SOURCE)
+		pa_context_get_source_output_info(impl->pa_context,
+				idx, source_output_info_cb, impl);
+	else
+		pa_context_get_sink_input_info(impl->pa_context,
+				idx, sink_input_info_cb, impl);
+}
+
 static pa_proplist* tunnel_new_proplist(struct impl *impl)
 {
 	pa_proplist *proplist = pa_proplist_new();
@@ -666,6 +811,8 @@ static int create_pulse_stream(struct impl *impl)
 
 	pa_threaded_mainloop_lock(impl->pa_mainloop);
 
+	pa_context_set_subscribe_callback(impl->pa_context, context_subscribe_cb, impl);
+
 	if (pa_threaded_mainloop_start(impl->pa_mainloop) < 0)
 		goto error_unlock;
 
@@ -724,6 +871,9 @@ static int create_pulse_stream(struct impl *impl)
 	if (impl->mode == MODE_SOURCE) {
 		bufferattr.fragsize = latency_bytes / 2;
 
+		pa_context_subscribe(impl->pa_context,
+				PA_SUBSCRIPTION_MASK_SOURCE_OUTPUT, NULL, impl);
+
 		res = pa_stream_connect_record(impl->pa_stream,
 				remote_node_target, &bufferattr,
 				PA_STREAM_DONT_MOVE |
@@ -735,6 +885,9 @@ static int create_pulse_stream(struct impl *impl)
 		bufferattr.minreq = bufferattr.tlength / 4;
 		bufferattr.prebuf = bufferattr.tlength;
 
+		pa_context_subscribe(impl->pa_context,
+				PA_SUBSCRIPTION_MASK_SINK_INPUT, NULL, impl);
+
 		res = pa_stream_connect_playback(impl->pa_stream,
 				remote_node_target, &bufferattr,
 				PA_STREAM_DONT_MOVE |
@@ -773,7 +926,7 @@ error_unlock:
 	pa_threaded_mainloop_unlock(impl->pa_mainloop);
 error:
 	pw_log_error("failed to connect: %s", pa_strerror(res));
-	return -res;
+	return err_to_res(res);
 }
 
 
@@ -980,6 +1133,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 
 	impl->module = module;
 	impl->context = context;
+	impl->main_loop = pw_context_get_main_loop(context);
 
 	spa_ringbuffer_init(&impl->ring);
 	impl->buffer = calloc(1, RINGBUFFER_SIZE);
