@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2019 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2019 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <errno.h>
 #include <stdio.h>
@@ -95,15 +75,15 @@ struct port {
 
 	struct pw_properties *props;
 
-	uint32_t change_mask_all;
+	uint64_t change_mask_all;
 	struct spa_port_info info;
 	struct spa_list param_list;
-#define IDX_EnumFormat	0
-#define IDX_Meta	1
-#define IDX_IO		2
-#define IDX_Format	3
-#define IDX_Buffers	4
-#define IDX_Latency	5
+#define PORT_EnumFormat	0
+#define PORT_Meta	1
+#define PORT_IO		2
+#define PORT_Format	3
+#define PORT_Buffers	4
+#define PORT_Latency	5
 #define N_PORT_PARAMS	6
 	struct spa_param_info params[N_PORT_PARAMS];
 
@@ -133,6 +113,7 @@ struct filter {
 	struct spa_node impl_node;
 	struct spa_hook_list hooks;
 	struct spa_callbacks callbacks;
+	struct spa_io_clock *clock;
 	struct spa_io_position *position;
 
 	struct {
@@ -142,12 +123,12 @@ struct filter {
 	struct spa_list port_list;
 	struct pw_map ports[2];
 
-	uint32_t change_mask_all;
+	uint64_t change_mask_all;
 	struct spa_node_info info;
 	struct spa_list param_list;
-#define IDX_PropInfo		0
-#define IDX_Props		1
-#define IDX_ProcessLatency	2
+#define NODE_PropInfo		0
+#define NODE_Props		1
+#define NODE_ProcessLatency	2
 #define N_NODE_PARAMS		3
 	struct spa_param_info params[N_NODE_PARAMS];
 
@@ -163,22 +144,22 @@ struct filter {
 
 	unsigned int disconnecting:1;
 	unsigned int disconnect_core:1;
-	unsigned int subscribe:1;
 	unsigned int draining:1;
 	unsigned int allow_mlock:1;
 	unsigned int warn_mlock:1;
 	unsigned int process_rt:1;
+	unsigned int driving:1;
 };
 
 static int get_param_index(uint32_t id)
 {
 	switch (id) {
 	case SPA_PARAM_PropInfo:
-		return IDX_PropInfo;
+		return NODE_PropInfo;
 	case SPA_PARAM_Props:
-		return IDX_Props;
+		return NODE_Props;
 	case SPA_PARAM_ProcessLatency:
-		return IDX_ProcessLatency;
+		return NODE_ProcessLatency;
 	default:
 		return -1;
 	}
@@ -188,17 +169,17 @@ static int get_port_param_index(uint32_t id)
 {
 	switch (id) {
 	case SPA_PARAM_EnumFormat:
-		return IDX_EnumFormat;
+		return PORT_EnumFormat;
 	case SPA_PARAM_Meta:
-		return IDX_Meta;
+		return PORT_Meta;
 	case SPA_PARAM_IO:
-		return IDX_IO;
+		return PORT_IO;
 	case SPA_PARAM_Format:
-		return IDX_Format;
+		return PORT_Format;
 	case SPA_PARAM_Buffers:
-		return IDX_Buffers;
+		return PORT_Buffers;
 	case SPA_PARAM_Latency:
-		return IDX_Latency;
+		return PORT_Latency;
 	default:
 		return -1;
 	}
@@ -475,6 +456,12 @@ static int impl_set_io(void *object, uint32_t id, void *data, size_t size)
 	pw_log_debug("%p: io %d %p/%zd", impl, id, data, size);
 
 	switch(id) {
+	case SPA_IO_Clock:
+		if (data && size >= sizeof(struct spa_io_clock))
+			impl->clock = data;
+		else
+			impl->clock = NULL;
+		break;
 	case SPA_IO_Position:
 		if (data && size >= sizeof(struct spa_io_position))
 			impl->position = data;
@@ -484,6 +471,7 @@ static int impl_set_io(void *object, uint32_t id, void *data, size_t size)
 			do_set_position, 1, NULL, 0, true, impl);
 		break;
 	}
+	impl->driving = impl->clock && impl->position && impl->position->clock.id == impl->clock->id;
 	pw_filter_emit_io_changed(&impl->this, NULL, id, data, size);
 
 	return 0;
@@ -1144,10 +1132,12 @@ static void proxy_error(void *_data, int seq, int res, const char *message)
 			PW_FILTER_STATE_ERROR, message);
 }
 
-static void proxy_bound(void *_data, uint32_t global_id)
+static void proxy_bound_props(void *_data, uint32_t global_id, const struct spa_dict *props)
 {
 	struct pw_filter *filter = _data;
 	filter->node_id = global_id;
+	if (props)
+		pw_properties_update(filter->properties, props);
 	filter_set_state(filter, PW_FILTER_STATE_PAUSED, NULL);
 }
 
@@ -1156,7 +1146,7 @@ static const struct pw_proxy_events proxy_events = {
 	.removed = proxy_removed,
 	.destroy = proxy_destroy,
 	.error = proxy_error,
-	.bound = proxy_bound,
+	.bound_props = proxy_bound_props,
 };
 
 static void on_core_error(void *_data, uint32_t id, int seq, int res, const char *message)
@@ -1202,6 +1192,8 @@ filter_new(struct pw_context *context, const char *name,
 	struct match match;
 	int res;
 
+	ensure_loop(context->main_loop, return NULL);
+
 	impl = calloc(1, sizeof(struct filter));
 	if (impl == NULL) {
 		res = -errno;
@@ -1366,21 +1358,58 @@ const char *pw_filter_state_as_string(enum pw_filter_state state)
 	return "invalid-state";
 }
 
+static int filter_disconnect(struct filter *impl)
+{
+	struct pw_filter *filter = &impl->this;
+	pw_log_debug("%p: disconnect", impl);
+
+	if (impl->disconnecting)
+		return -EBUSY;
+
+	impl->disconnecting = true;
+
+	if (filter->proxy) {
+		pw_proxy_destroy(filter->proxy);
+		filter->proxy = NULL;
+	}
+	if (impl->disconnect_core) {
+		impl->disconnect_core = false;
+		spa_hook_remove(&filter->core_listener);
+		spa_list_remove(&filter->link);
+		pw_core_disconnect(filter->core);
+		filter->core = NULL;
+	}
+	return 0;
+}
+
+static void free_port(struct filter *impl, struct port *port)
+{
+	spa_list_remove(&port->link);
+	spa_node_emit_port_info(&impl->hooks, port->direction, port->id, NULL);
+	pw_map_remove(&impl->ports[port->direction], port->id);
+	clear_buffers(port);
+	clear_params(impl, port, SPA_ID_INVALID);
+	pw_properties_free(port->props);
+	free(port);
+}
+
 SPA_EXPORT
 void pw_filter_destroy(struct pw_filter *filter)
 {
 	struct filter *impl = SPA_CONTAINER_OF(filter, struct filter, this);
 	struct port *p;
 
+	ensure_loop(impl->context->main_loop, return);
+
 	pw_log_debug("%p: destroy", filter);
 
 	pw_filter_emit_destroy(filter);
 
 	if (!impl->disconnecting)
-		pw_filter_disconnect(filter);
+		filter_disconnect(impl);
 
 	spa_list_consume(p, &impl->port_list, link)
-		pw_filter_remove_port(p->user_data);
+		free_port(impl, p);
 
 	if (filter->core) {
 		spa_hook_remove(&filter->core_listener);
@@ -1423,6 +1452,9 @@ void pw_filter_add_listener(struct pw_filter *filter,
 			    void *data)
 {
 	struct filter *impl = SPA_CONTAINER_OF(filter, struct filter, this);
+
+	ensure_loop(impl->context->main_loop);
+
 	spa_hook_list_append(&filter->listener_list, listener, events, data);
 	if (events->process && impl->rt_callbacks.funcs == NULL) {
 		impl->rt_callbacks = SPA_CALLBACKS_INIT(events, data);
@@ -1471,6 +1503,8 @@ int pw_filter_update_properties(struct pw_filter *filter, void *port_data, const
 	struct port *port = SPA_CONTAINER_OF(port_data, struct port, user_data);
 	int changed = 0;
 
+	ensure_loop(impl->context->main_loop, return -EIO);
+
 	if (port_data) {
 		changed = pw_properties_update(port->props, dict);
 		port->info.props = &port->props->dict;
@@ -1508,6 +1542,11 @@ pw_filter_connect(struct pw_filter *filter,
 	uint32_t i;
 	struct spa_dict_item items[1];
 
+	ensure_loop(impl->context->main_loop, return -EIO);
+
+	if (filter->proxy != NULL || filter->state != PW_FILTER_STATE_UNCONNECTED)
+		return -EBUSY;
+
 	pw_log_debug("%p: connect", filter);
 	impl->flags = flags;
 
@@ -1530,9 +1569,9 @@ pw_filter_connect(struct pw_filter *filter,
 	impl->info.max_output_ports = UINT32_MAX;
 	impl->info.flags = impl->process_rt ? SPA_NODE_FLAG_RT : 0;
 	impl->info.props = &filter->properties->dict;
-	impl->params[IDX_PropInfo] = SPA_PARAM_INFO(SPA_PARAM_PropInfo, 0);
-	impl->params[IDX_Props] = SPA_PARAM_INFO(SPA_PARAM_Props, SPA_PARAM_INFO_WRITE);
-	impl->params[IDX_ProcessLatency] = SPA_PARAM_INFO(SPA_PARAM_ProcessLatency, 0);
+	impl->params[NODE_PropInfo] = SPA_PARAM_INFO(SPA_PARAM_PropInfo, 0);
+	impl->params[NODE_Props] = SPA_PARAM_INFO(SPA_PARAM_Props, SPA_PARAM_INFO_WRITE);
+	impl->params[NODE_ProcessLatency] = SPA_PARAM_INFO(SPA_PARAM_ProcessLatency, 0);
 	impl->info.params = impl->params;
 	impl->info.n_params = N_NODE_PARAMS;
 	impl->info.change_mask = impl->change_mask_all;
@@ -1543,6 +1582,8 @@ pw_filter_connect(struct pw_filter *filter,
 	}
 
 	impl->disconnecting = false;
+	impl->draining = false;
+	impl->driving = false;
 	filter_set_state(filter, PW_FILTER_STATE_CONNECTING, NULL);
 
 	if (flags & PW_FILTER_FLAG_DRIVER)
@@ -1596,22 +1637,8 @@ SPA_EXPORT
 int pw_filter_disconnect(struct pw_filter *filter)
 {
 	struct filter *impl = SPA_CONTAINER_OF(filter, struct filter, this);
-
-	pw_log_debug("%p: disconnect", filter);
-	impl->disconnecting = true;
-
-	if (filter->proxy) {
-		pw_proxy_destroy(filter->proxy);
-		filter->proxy = NULL;
-	}
-	if (impl->disconnect_core) {
-		impl->disconnect_core = false;
-		spa_hook_remove(&filter->core_listener);
-		spa_list_remove(&filter->link);
-		pw_core_disconnect(filter->core);
-		filter->core = NULL;
-	}
-	return 0;
+	ensure_loop(impl->context->main_loop, return -EIO);
+	return filter_disconnect(impl);
 }
 
 static void add_port_params(struct filter *impl, struct port *port)
@@ -1693,6 +1720,8 @@ void *pw_filter_add_port(struct pw_filter *filter,
 	struct port *p;
 	const char *str;
 
+	ensure_loop(impl->context->main_loop, return NULL);
+
 	if (props == NULL)
 		props = pw_properties_new(NULL, NULL);
 	if (props == NULL)
@@ -1712,12 +1741,12 @@ void *pw_filter_add_port(struct pw_filter *filter,
 		p->info.flags |= SPA_PORT_FLAG_CAN_ALLOC_BUFFERS;
 	p->info.props = &p->props->dict;
 	p->change_mask_all |= SPA_PORT_CHANGE_MASK_PARAMS;
-	p->params[IDX_EnumFormat] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, 0);
-	p->params[IDX_Meta] = SPA_PARAM_INFO(SPA_PARAM_Meta, 0);
-	p->params[IDX_IO] = SPA_PARAM_INFO(SPA_PARAM_IO, 0);
-	p->params[IDX_Format] = SPA_PARAM_INFO(SPA_PARAM_Format, SPA_PARAM_INFO_WRITE);
-	p->params[IDX_Buffers] = SPA_PARAM_INFO(SPA_PARAM_Buffers, 0);
-	p->params[IDX_Latency] = SPA_PARAM_INFO(SPA_PARAM_Latency, SPA_PARAM_INFO_WRITE);
+	p->params[PORT_EnumFormat] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, 0);
+	p->params[PORT_Meta] = SPA_PARAM_INFO(SPA_PARAM_Meta, 0);
+	p->params[PORT_IO] = SPA_PARAM_INFO(SPA_PARAM_IO, 0);
+	p->params[PORT_Format] = SPA_PARAM_INFO(SPA_PARAM_Format, SPA_PARAM_INFO_WRITE);
+	p->params[PORT_Buffers] = SPA_PARAM_INFO(SPA_PARAM_Buffers, 0);
+	p->params[PORT_Latency] = SPA_PARAM_INFO(SPA_PARAM_Latency, SPA_PARAM_INFO_WRITE);
 	p->info.params = p->params;
 	p->info.n_params = N_PORT_PARAMS;
 
@@ -1755,16 +1784,9 @@ int pw_filter_remove_port(void *port_data)
 	struct port *port = SPA_CONTAINER_OF(port_data, struct port, user_data);
 	struct filter *impl = port->filter;
 
-	spa_node_emit_port_info(&impl->hooks, port->direction, port->id, NULL);
-
-	spa_list_remove(&port->link);
-	pw_map_remove(&impl->ports[port->direction], port->id);
-
-	clear_buffers(port);
-	clear_params(impl, port, SPA_ID_INVALID);
-	pw_properties_free(port->props);
-	free(port);
+	ensure_loop(impl->context->main_loop, return -EIO);
 
+	free_port(impl, port);
 	return 0;
 }
 
@@ -1772,6 +1794,10 @@ SPA_EXPORT
 int pw_filter_set_error(struct pw_filter *filter,
 			int res, const char *error, ...)
 {
+	struct filter *impl = SPA_CONTAINER_OF(filter, struct filter, this);
+
+	ensure_loop(impl->context->main_loop, return -EIO);
+
 	if (res < 0) {
 		va_list args;
 		char *value;
@@ -1802,6 +1828,8 @@ int pw_filter_update_params(struct pw_filter *filter,
 	struct port *port;
 	int res;
 
+	ensure_loop(impl->context->main_loop, return -EIO);
+
 	pw_log_debug("%p: update params", filter);
 
 	port = port_data ? SPA_CONTAINER_OF(port_data, struct port, user_data) : NULL;
@@ -1821,6 +1849,10 @@ int pw_filter_update_params(struct pw_filter *filter,
 SPA_EXPORT
 int pw_filter_set_active(struct pw_filter *filter, bool active)
 {
+	struct filter *impl = SPA_CONTAINER_OF(filter, struct filter, this);
+
+	ensure_loop(impl->context->main_loop, return -EIO);
+
 	pw_log_debug("%p: active:%d", filter, active);
 	return 0;
 }
@@ -1844,25 +1876,6 @@ int pw_filter_get_time(struct pw_filter *filter, struct pw_time *time)
 	return 0;
 }
 
-static int
-do_process(struct spa_loop *loop,
-                 bool async, uint32_t seq, const void *data, size_t size, void *user_data)
-{
-	struct filter *impl = user_data;
-	int res = impl_node_process(impl);
-	return spa_node_call_ready(&impl->callbacks, res);
-}
-
-static inline int call_trigger(struct filter *impl)
-{
-	int res = 0;
-	if (SPA_FLAG_IS_SET(impl->flags, PW_FILTER_FLAG_DRIVER)) {
-		res = pw_loop_invoke(impl->context->data_loop,
-			do_process, 1, NULL, 0, false, impl);
-	}
-	return res;
-}
-
 SPA_EXPORT
 struct pw_buffer *pw_filter_dequeue_buffer(void *port_data)
 {
@@ -1894,7 +1907,7 @@ int pw_filter_queue_buffer(void *port_data, struct pw_buffer *buffer)
 	if ((res = push_queue(p, &p->queued, b)) < 0)
 		return res;
 
-	return call_trigger(impl);
+	return res;
 }
 
 SPA_EXPORT
@@ -1960,3 +1973,51 @@ int pw_filter_flush(struct pw_filter *filter, bool drain)
 			drain ? do_drain : do_flush, 1, NULL, 0, true, impl);
 	return 0;
 }
+
+SPA_EXPORT
+bool pw_filter_is_driving(struct pw_filter *filter)
+{
+	struct filter *impl = SPA_CONTAINER_OF(filter, struct filter, this);
+	return impl->driving;
+}
+
+static int
+do_trigger_process(struct spa_loop *loop,
+                 bool async, uint32_t seq, const void *data, size_t size, void *user_data)
+{
+	struct filter *impl = user_data;
+	int res = impl_node_process(impl);
+	return spa_node_call_ready(&impl->callbacks, res);
+}
+
+static int do_trigger_request_process(struct spa_loop *loop,
+                 bool async, uint32_t seq, const void *data, size_t size, void *user_data)
+{
+	struct filter *impl = user_data;
+	uint8_t buffer[1024];
+	struct spa_pod_builder b = { 0 };
+
+	spa_pod_builder_init(&b, buffer, sizeof(buffer));
+	spa_node_emit_event(&impl->hooks,
+			spa_pod_builder_add_object(&b,
+				SPA_TYPE_EVENT_Node, SPA_NODE_EVENT_RequestProcess));
+	return 0;
+}
+
+SPA_EXPORT
+int pw_filter_trigger_process(struct pw_filter *filter)
+{
+	struct filter *impl = SPA_CONTAINER_OF(filter, struct filter, this);
+	int res = 0;
+
+	pw_log_trace_fp("%p", impl);
+
+	if (!impl->driving) {
+		res = pw_loop_invoke(impl->context->main_loop,
+			do_trigger_request_process, 1, NULL, 0, false, impl);
+	} else {
+		res = pw_loop_invoke(impl->context->data_loop,
+			do_trigger_process, 1, NULL, 0, false, impl);
+	}
+	return res;
+}
