@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <errno.h>
 #include <string.h>
@@ -779,9 +759,9 @@ int pw_impl_link_prepare(struct pw_impl_link *this)
 {
 	struct impl *impl = SPA_CONTAINER_OF(this, struct impl, this);
 
-	pw_log_debug("%p: prepared:%d preparing:%d in_active:%d out_active:%d",
+	pw_log_debug("%p: prepared:%d preparing:%d in_active:%d out_active:%d passive:%u",
 			this, this->prepared, this->preparing,
-			impl->inode->active, impl->onode->active);
+			impl->inode->active, impl->onode->active, this->passive);
 
 	if (!impl->inode->active || !impl->onode->active)
 		return 0;
@@ -1172,6 +1152,7 @@ struct pw_impl_link *pw_context_create_link(struct pw_context *context,
 	struct impl *impl;
 	struct pw_impl_link *this;
 	struct pw_impl_node *input_node, *output_node;
+	const char *str;
 	int res;
 
 	if (output == input)
@@ -1221,7 +1202,13 @@ struct pw_impl_link *pw_context_create_link(struct pw_context *context,
 	this->input = input;
 
 	/* passive means that this link does not make the nodes active */
-	this->passive = pw_properties_get_bool(properties, PW_KEY_LINK_PASSIVE, false);
+	str = pw_properties_get(properties, PW_KEY_LINK_PASSIVE);
+	this->passive = str ? spa_atob(str) :
+		(output->passive && input_node->can_suspend) ||
+		(input->passive && output_node->can_suspend) ||
+		(input->passive && output->passive);
+	if (this->passive && str == NULL)
+		 pw_properties_set(properties, PW_KEY_LINK_PASSIVE, "true");
 
 	spa_hook_list_init(&this->listener_list);
 
@@ -1272,10 +1259,9 @@ struct pw_impl_link *pw_context_create_link(struct pw_context *context,
 		     output_node, output->port_id, this->rt.out_mix.port.port_id,
 		     input_node, input->port_id, this->rt.in_mix.port.port_id);
 
-	if (asprintf(&this->name, "%d.%d -> %d.%d",
+	this->name = spa_aprintf("%d.%d -> %d.%d",
 			output_node->info.id, output->port_id,
-			input_node->info.id, input->port_id) < 0)
-		this->name = NULL;
+			input_node->info.id, input->port_id);
 	pw_log_info("(%s) (%s) -> (%s)", this->name, output_node->name, input_node->name);
 
 	pw_impl_port_emit_link_added(output, this);
